from jinja2 import Environment, FileSystemLoader
from .metrics import log_metrics
from huble.sklearn.process.templating import return_function as preprocess_generate
from huble.sklearn.train.templates import return_function as train_generate

from collections import defaultdict
 
class Graph:
    def __init__(self, vertices):
        self.graph = defaultdict(list) 
        self.V = vertices 

    def addEdge(self, u, v):
        self.graph[u].append(v)
 
    def topologicalSortUtil(self, v, visited, stack):
        visited[v] = True
        for i in self.graph[v]:
            if visited[i] == False:
                self.topologicalSortUtil(i, visited, stack)
        stack.append(v)

    def topologicalSort(self):
        visited = [False]*self.V
        stack = []
        for i in range(self.V):
            if visited[i] == False:
                self.topologicalSortUtil(i, visited, stack)
        return(stack[::-1]) 
 
def generate_file(graph, colab=False):
    g = Graph(len(graph['nodes']))
    map = {}
    j=0
    while j <(len(graph['nodes'])):
        for i in graph['nodes']:
            map[(i['id'])]=j
            j+=1
    map2 = {y: x for x, y in map.items()}
    for i in graph['edges']:
        g.addEdge(map[i['source']], map[i['target']])
    res=g.topologicalSort()
    steps=[]
    for i in res:
        steps.append(map2[i])
    steps_list={}
    for i in steps:
        for j in range(len(graph['nodes'])):
            if i == graph['nodes'][j]['id']:
                steps_list[(graph['nodes'][j]['data']['name'])]=graph['nodes'][j]['data']['node_type']

    with open("/content/output.py", "w") as f:
        f.write("import huble\n")
        
        for i in steps_list:
            for node in graph['nodes']:
                if i == node['data']['name']:
                    if steps_list[i]=='preprocess':
                        f.write(preprocess_generate(node))
                    elif steps_list[i] == 'model':
                        f.write(train_generate(node))
                    elif steps_list[i] == 'essential':
                        f.write("essentials")
                    elif steps_list[i] == 'evaluate_model':
                        f.write("evaluate model")
    








# def generate_file(url,graph,colab=False):

#     preprocess_template = preprocess_generate(graph)
#     train_template = temp_logistic_regression()
#     #output_template = preprocess_template + train_template
#     with open("/content/output.py", "w") as f:
#         f.write("import huble\n")
#         f.write("from sklearn.model_selection import train_test_split\n")
#         f.write(f"data=huble.util.load_dataset({url})")
#         f.write(preprocess_template)
#         f.write("\nX = data.drop(columns = ['survived'],axis = 1)\n")
#         f.write("y = data['survived']\n")
#         f.write("X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)\n")
#         f.write(train_template)
#         f.write("\nmodel.fit(X,y)\n")
#         f.write("y_pred = model.predict(X_test)\n")
#         #metrics template
#         f.write("metrics = huble.sklearn.metrics.log_metrics(y_test, y_pred,'classification')\n")
#         f.write("huble.sklearn.metrics.upload(metrics)\n")
        

#     #print(output_template)




import pandas as pd
import requests


def load_dataset(url) -> pd.DataFrame:
    local_filename = url.split('/')[-1]
    # NOTE the stream=True parameter below
    
    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        with open(local_filename, "wb") as f:
            for chunk in r.iter_content(chunk_size=8192):
                f.write(chunk)
    return pd.read_csv(local_filename)



import pandas as pd
import woodwork as ww

def parse_dataset(dataset):
    # parse dataset using woodwork and return a dictionary with column names as keys and logical column types as values
    ww_df = ww.init(dataset)
    logical_types = ww_df.logical_types
    logical_types_dict = {}
    for col in logical_types:
        logical_types_dict[col] = logical_types[col]
    return logical_types_dict

  # dict = {}
    # dict["rows"] = len(dataset)
    # dict["columns"] = []
    # for column in dataset.columns:
    #     if (
    #         column.lower() == "location"
    #         or column.lower() == "city"
    #         or column.lower() == "country"
    #         or column.lower() == "state"
    #     ):
    #         dict["columns"].append({"name": column, "type": "location"})
    #     if (
    #         column
    #         in dataset.select_dtypes(
    #             include=["int64", "float64", "int32", "float32"]
    #         ).columns.values
    #     ):
    #         dict["columns"].append({"name": column, "type": "numeric"})
    #     elif column in dataset.select_dtypes(include=["object"]).columns.values:
    #         if len(dataset[column].unique()) < len(dataset) * 0.1:
    #             dict["columns"].append({"name": column, "type": "categorical"})
    #             continue
    #         try:
    #             pd.to_datetime(dataset[column], dayfirst=True)
    #             dict["columns"].append({"name": column, "type": "datetime"})
    #             continue
    #         except:
    #             dict["columns"].append({"name": column, "type": "text"})
    # return dict
      


    



import re
from re import sub
import sklearn
from sklearn.impute import SimpleImputer
import pandas as pd
from scipy import stats
import numpy as np
# import AutoClean
# from AutoClean import AutoClean

def remove_nan_values(**params):
    data = params['data']
    data = data.dropna(**params["parameters"])
    return data


def replace_nan_values(**params):
    imputer = SimpleImputer(**params["parameters"])
    data = params['data']
    data[(params['column'])] = imputer.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


def drop_rows_columns(**params):
    data = params['data']
    data = data.drop(**params["parameters"])
    return data


def remove_outliers(**params):
    data = params['data']
    for column in params["columns"]:
        data = data[
            (np.abs(stats.zscore(data[(params['column'])])) < 3)
        ]
    return data


def drop_duplicates(**params):
    data = params['data']
    data = data.drop_duplicates(**params["parameters"])
    return data


def change_data_type(**params):
    data = params['data']
    data[(params['column'])] = data[(params['column'])].astype(**params["parameters"])
    return data


def round_data(**params):
    data = params['data']
    data = data.round(**params["parameters"])
    return data


def filter_dataframe(**params):
    data = params['data']
    data = data.filter(**params["parameters"])
    return data


def truncate_dataframe(**params):
    data = params['data']
    data = data.truncate(**params["parameters"])
    return data


def sort_values(**params):
    data = params['data']
    data = data.sort_values(**params["parameters"])
    return data


def transpose(**params):
    data = params['data']
    data = data.transpose()
    return data


def min_max_scale(**params):
    data = params['data']
    scaler = sklearn.preprocessing.MinMaxScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


def max_abs_scale(**params):
    data = params['data']
    scaler = sklearn.preprocessing.MaxAbsScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


def robust_scale(**params):
    data = params['data']
    scaler = sklearn.preprocessing.RobustScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


def standard_scale(**params):
    data = params['data']
    scaler = sklearn.preprocessing.StandardScaler(**params["parameters"])
    data[(params['column'])] = scaler.fit_transform(data[(params['column'])].values.reshape(-1,1))[:,0]
    return data


def normalize(**params):
    data = params['data']
    for column in params['columns']:
        params["X"]=column
        data = sklearn.preprocessing.normalize(**params["parameters"])
    return data


def ordinal_encode(**params):
    data = params['data']
    data_columns = params["columns"]
    enc = sklearn.preprocessing.OrdinalEncoder(**params["parameters"])
    data[data_columns] = enc.fit_transform(data[data_columns])
    return data


def one_hot_encode(**params):
    data = params['data']
    data_columns = params["columns"]
    le = sklearn.preprocessing.LabelEncoder()
    data[data_columns] = data[data_columns].apply(lambda col: le.fit_transform(col))
    enc = sklearn.preprocessing.OneHotEncoder(**params["parameters"])
    array_hot_encoded = enc.fit_transform(data[data_columns])
    data_hot_encoded = pd.DataFrame(array_hot_encoded, index=data.index)
    data_other_cols = data.drop(columns=data_columns)
    data_out = pd.concat([data_hot_encoded, data_other_cols], axis=1)
    print(data_out)
    return data_out
   

def remove_mismatch_data(**params):
    data = params['data']
    exceptions = params['parameters']['exceptions']
    for col in data : 
        if col in exceptions : continue
        data.reset_index(drop=True, inplace=True)
        s = [False]*len(data[col])
        for i,cell in enumerate(data[col]) : 
            try : n = int(cell)
            except : s[i] = True
        t = s.count(True)
        f = s.count(False)
        st = False
        if(t>f) : st = True
        remove = [i for i in range(len(data[col])) if s[i]!=st]
        data.drop(remove,axis=0,inplace=True)
    return data

def merge(**params):
    data = params['data']
    df = data.merge(**params['parameters'])
    return df

def rename_columns(**params):
    data = params['data']
    dict = params['dict']
    axis = params['axis']
    data = data.rename(dict, axis)
    return data

def select_column(**params):
    data = params['data']
    df = data.select_dtypes(**params['parameters'])
    return df

def clean_column_names(**params):
    data = params['data']
    match=r"[\]\[\,\{\}\"\:]+"
    data.rename(columns=lambda x: re.sub(match, "", str(x)))
    return data

# def clean_data(**params):
#     data = params['data']
#     df = AutoClean(data)
#     return df.output

def clip(**params):
    data = params['data']
    data = data.clip(**params['paramters'])
    return data

def split_data(**params):
    data = params['data']
    df_shuffled = data.sample(frac=1)
    df_splits = np.array_split(df_shuffled, 2)
    return df_splits




from jinja2 import Environment, FileSystemLoader
from .templates import return_function 
import os


def generate_script(node):
    """
    Generate script for training data.
    """
    output =  return_function(node)
    return output



import requests


def get_experiment_details(version_id):
    response = requests.get(
        f"http://localhost:8000/verisons/{version_id}/",
    )
    response = response.json()
    graph = response['experiment']['pipeline']['pipelineJSON']
    url = f"https://ipfs.filebase.io/ipfs/{response['experiment']['model']['filePath']}"
    return graph,url


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['huble',
 'huble.sklearn',
 'huble.sklearn.automl',
 'huble.sklearn.deploy',
 'huble.sklearn.metrics',
 'huble.sklearn.process',
 'huble.sklearn.train',
 'huble.util']

package_data = \
{'': ['*'],
 'huble.sklearn': ['process/template/*',
                   'temp/*',
                   'templates/*',
                   'train/template/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'auto-sklearn==0.14.7',
 'black>=22.8.0,<23.0.0',
 'pandas>=1.3.5,<2.0.0',
 'scikit-learn==0.24.2',
 'scipy>=1.7.0,<1.8.0',
 'woodwork>=0.21.1,<0.22.0']

setup_kwargs = {
    'name': 'huble',
    'version': '0.1.89',
    'description': '',
    'long_description': '',
    'author': 'Rugz007',
    'author_email': 'rugvedsomwanshi007@gmail.com',
    'maintainer': 'Arjit Agarwal',
    'maintainer_email': 'arjitagarwal123@gmail.com',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.1,<3.10',
}


setup(**setup_kwargs)



from jinja2 import Environment, FileSystemLoader
# import templating
from .templating import return_function
import os




def generate_script(node):
    """
    Generate script for processing data.
    # """
   
    output =  return_function(node)
    return output

    

# def aa():
#     test = """{"nodes": [{"id": "999",
#    "type": "custom",
#    "position": {"x": 261, "y": 134},
#    "data": {"name": "Raw Data",
#     "value": "Raw Data",
#     "color": "pink.400",
#     "isInput": true,
#     "parameters": {"Dataset": "Test"},
#     "node_type": "preprocess"},
#    "width": 400,
#    "height": 76,
#    "selected": false,
#    "positionAbsolute": {"x": 261, "y": 134},
#    "dragging": false,
#    "target": "1"},
#   {"id": "998",
#    "type": "custom",
#    "position": {"x": 462.7212309761019, "y": 816.3344123088411},
#    "data": {"name": "Clean Data",
#     "value": "Clean Data",
#     "color": "green.400",
#     "isOutput": true,
#     "parameters": {"Dataset": "Test"},
#     "node_type": "preprocess"},
#    "width": 400,
#    "height": 76,
#    "selected": false,
#    "positionAbsolute": {"x": 462.7212309761019, "y": 816.3344123088411},
#    "dragging": false,
#    "source": "5"},
#   {"id": "1",
#    "type": "custom",
#    "position": {"x": 482.9480535053194, "y": 273.0009967292071},
#    "data": {"name": "Remove NAN values",
#     "value": "Remove NAN values",
#     "parameters": {"axis": 0, "how": "any", "subset": [], "inplace": false},
#     "node_type": "preprocess",
#     "color": "orange.400"},
#    "width": 400,
#    "height": 76,
#    "selected": false,
#    "positionAbsolute": {"x": 482.9480535053194, "y": 273.0009967292071},
#    "dragging": false,
#    "source": "999",
#    "target": "2"},
#   {"id": "2",
#    "type": "custom",
#    "position": {"x": 445.882683707091, "y": 423.57906153451086},
#    "data": {"name": "Dropping rows or columns",
#     "value": "Dropping rows or columns",
#     "parameters": {"labels": [{"value": "home.dest", "label": "home.dest"},
#       {"value": "name", "label": "name"}],
#      "axis": 0,
#      "index": [],
#      "columns": [],
#      "inplace": false,
#      "errors": "raise"},
#     "node_type": "preprocess",
#     "color": "teal.400"},
#    "width": 400,
#    "height": 76,
#    "source": "1",
#    "target": "4",
#    "selected": false,
#    "positionAbsolute": {"x": 445.882683707091, "y": 423.57906153451086},
#    "dragging": false},
#   {"id": "4",
#    "type": "custom",
#    "position": {"x": 412.2921923274464, "y": 554.4661486345054},
#    "data": {"name": "One Hot Encoding",
#     "value": "One Hot Encoding",
#     "parameters": {"columns": [{"value": "sex", "label": "sex"}],
#      "categories": "auto",
#      "dtype": "np.float",
#      "handle_unknown": "ignore",
#      "sparse": true,
#      "min_frequency": "null",
#      "max_categories": "null"},
#     "node_type": "preprocess",
#     "color": "teal.400"},
#    "width": 400,
#    "height": 76,
#    "selected": false,
#    "positionAbsolute": {"x": 412.2921923274464, "y": 554.4661486345054},
#    "dragging": false,
#    "source": "2",
#    "target": "5"},
#   {"id": "5",
#    "type": "custom",
#    "position": {"x": 398.3926786531106, "y": 676.086893284943},
#    "data": {"name": "Remove Outliers",
#     "value": "Remove Outliers",
#     "parameters": {"columns": [{"value": "fare", "label": "fare"}]},
#     "node_type": "preprocess",
#     "color": "yellow.400"},
#    "width": 400,
#    "height": 76,
#    "source": "4",
#    "selected": true,
#    "positionAbsolute": {"x": 398.3926786531106, "y": 676.086893284943},
#    "dragging": false,
#    "target": "998"}],
#  "edges": [{"source": "999",
#    "sourceHandle": null,
#    "target": "1",
#    "targetHandle": null,
#    "type": "custom",
#    "data": {"sourceColor": "pink.400", "targetColor": "orange.400"},
#    "id": "reactflow__edge-999-1"},
#   {"source": "1",
#    "sourceHandle": null,
#    "target": "2",
#    "targetHandle": null,
#    "type": "custom",
#    "data": {"sourceColor": "orange.400", "targetColor": "teal.400"},
#    "id": "reactflow__edge-1-2"},
#   {"source": "2",
#    "sourceHandle": null,
#    "target": "3",
#    "targetHandle": null,
#    "type": "custom",
#    "data": {"sourceColor": "teal.400", "targetColor": "#805AD5"},
#    "id": "reactflow__edge-2-3"},
#   {"source": "3",
#    "sourceHandle": null,
#    "target": "998",
#    "targetHandle": null,
#    "type": "custom",
#    "data": {"sourceColor": "#805AD5", "targetColor": "green.400"},
#    "id": "reactflow__edge-3-998"},
#   {"source": "2",
#    "sourceHandle": null,
#    "target": "4",
#    "targetHandle": null,
#    "type": "custom",
#    "data": {"sourceColor": "teal.400", "targetColor": "teal.400"},
#    "id": "reactflow__edge-2-4"},
#   {"source": "4",
#    "sourceHandle": null,
#    "target": "5",
#    "targetHandle": null,
#    "type": "custom",
#    "data": {"sourceColor": "teal.400", "targetColor": "yellow.400"},
#    "id": "reactflow__edge-4-5"},
#   {"source": "5",
#    "sourceHandle": null,
#    "target": "998",
#    "targetHandle": null,
#    "type": "custom",
#    "data": {"sourceColor": "yellow.400", "targetColor": "green.400"},
#    "id": "reactflow__edge-5-998"}]}"""
#     import json
#     graph = json.loads(test)

#     res=generate_script(graph)
#     print(res)

